# Generated by Django 2.2.7 on 2019-11-26 19:08

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="RefreshToken",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("token", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="AccessToken",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("token", models.CharField(blank=True, max_length=255, null=True)),
                ("expires_at", models.DateTimeField()),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
    ]



from datetime import timedelta

import requests
from django.utils import timezone

from accounts.settings import accounts_settings


def authenticated_request(
    user,
    method,
    url,
    params=None,
    data=None,
    headers=None,
    cookies=None,
    files=None,
    auth=None,
    timeout=None,
    allow_redirects=True,
    proxies=None,
    hooks=None,
    stream=None,
    verify=None,
    cert=None,
    json=None,
):
    """
    Helper method to make an authenticated request using the user's access token
    NOTE be ABSOLUTELY sure you only make a request to Penn Labs products, otherwise
    you will expose user's access tokens to the URL you provide and bad things will
    happen
    """

    # Access token is expired. Try to refresh access token
    if user.accesstoken.expires_at < timezone.now():
        if not _refresh_access_token(user):
            # Couldn't update the user's access token. Return a response with a 403 status code
            # as if the user didn't have access to the requested resource
            response = requests.models.Response
            response.status_code = 403
            return response

    # Update Headers
    headers = {} if headers is None else headers
    headers["Authorization"] = f"Bearer {user.accesstoken.token}"

    # Make the request
    # We're only using a session to provide an easy wrapper to define the http method
    # GET, POST, etc in the method call.
    s = requests.Session()
    return s.request(
        method=method,
        url=url,
        params=params,
        data=data,
        headers=headers,
        cookies=cookies,
        files=files,
        auth=None,
        timeout=None,
        allow_redirects=allow_redirects,
        proxies=proxies,
        hooks=hooks,
        stream=stream,
        verify=verify,
        cert=cert,
        json=json,
    )


def _refresh_access_token(user):
    """
    Helper method to update a user's access token. Should be used when a user's
    access token has expired, but still has a valid refresh token.

    Returns:
        bool: true if the access token is updated, false otherwise.
    """
    body = {
        "grant_type": "refresh_token",
        "client_id": accounts_settings.CLIENT_ID,
        "client_secret": accounts_settings.CLIENT_SECRET,
        "refresh_token": user.refreshtoken.token,
    }
    try:
        data = requests.post(url=accounts_settings.PLATFORM_URL + "/accounts/token/", data=body)
        if data.status_code == 200:  # Access token refreshed successfully
            data = data.json()
            # Update Access token
            user.accesstoken.token = data["access_token"]
            user.accesstoken.expires_at = timezone.now() + timedelta(seconds=data["expires_in"])
            user.accesstoken.save()

            # Update Refresh Token
            user.refreshtoken.token = data["refresh_token"]
            user.refreshtoken.save()

            return True
    except requests.exceptions.RequestException:  # Can't connect to platform
        return False
    return False



from django.conf import settings
from django.db import models


class AccessToken(models.Model):
    token = models.CharField(max_length=255, blank=True, null=True)
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    expires_at = models.DateTimeField()

    def __str__(self):
        return str(self.token)


class RefreshToken(models.Model):
    token = models.CharField(max_length=255, blank=True, null=True)
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)

    def __str__(self):
        return str(self.token)



from django.contrib import auth
from django.http import HttpResponseRedirect, HttpResponseServerError
from django.http.response import HttpResponseBadRequest
from django.shortcuts import redirect
from django.urls import reverse
from django.views import View
from requests_oauthlib import OAuth2Session

from accounts.settings import accounts_settings


def get_redirect_uri(request):
    """
    Determine the redirect URI using either an environment variable or the request.
    """
    if accounts_settings.REDIRECT_URI:
        return accounts_settings.REDIRECT_URI
    return request.build_absolute_uri(reverse("accounts:callback"))


class LoginView(View):
    """
    Log in the user and redirect to next query parameter
    """

    def get(self, request):
        return_to = request.GET.get("next")
        if not return_to:
            return HttpResponseBadRequest("Invalid next parameter")
        request.session["next"] = return_to
        if not request.user.is_authenticated:
            platform = OAuth2Session(
                accounts_settings.CLIENT_ID,
                scope=accounts_settings.SCOPE,
                redirect_uri=get_redirect_uri(request),
            )
            authorization_url, state = platform.authorization_url(
                accounts_settings.PLATFORM_URL + "/accounts/authorize/"
            )
            response = redirect(authorization_url)
            request.session["state"] = state
            return response
        return redirect(request.session.pop("next", "/"))


class CallbackView(View):
    """
    View where the the user is redirected to from platform with the
    query parameter code being that user's Authorization Code
    """

    def get(self, request):
        response = HttpResponseRedirect(request.session.pop("next"))
        state = request.session.pop("state")
        platform = OAuth2Session(
            accounts_settings.CLIENT_ID, redirect_uri=get_redirect_uri(request), state=state
        )

        # Get the user's access and refresh tokens
        token = platform.fetch_token(
            accounts_settings.PLATFORM_URL + "/accounts/token/",
            client_secret=accounts_settings.CLIENT_SECRET,
            authorization_response=request.build_absolute_uri(),
        )

        # Use the access token to log in the user using information from platform
        platform = OAuth2Session(accounts_settings.CLIENT_ID, token=token)
        introspect_url = accounts_settings.PLATFORM_URL + "/accounts/introspect/"
        platform_request = platform.post(introspect_url, data={"token": token["access_token"]})
        if platform_request.status_code == 200:  # Connected to platform successfully
            user_props = platform_request.json()["user"]
            user_props["token"] = token
            user = auth.authenticate(request, remote_user=user_props)
            if user:
                auth.login(request, user)
                return response
        return HttpResponseServerError()


class LogoutView(View):
    """
    Log out the user and redirect to next query parameter
    """

    def get(self, request):
        auth.logout(request)
        return redirect(request.GET.get("next", "/"))



import os

from django.conf import settings


USER_SETTINGS = getattr(settings, "PLATFORM_ACCOUNTS", {})

DEFAULTS = {
    "CLIENT_ID": os.environ.get("LABS_CLIENT_ID"),
    "CLIENT_SECRET": os.environ.get("LABS_CLIENT_SECRET"),
    "REDIRECT_URI": os.environ.get("LABS_REDIRECT_URI"),
    "SCOPE": ["read", "introspection"],
    "PLATFORM_URL": "https://platform.pennlabs.org",
    "ADMIN_PERMISSION": "example_admin",
    "CUSTOM_ADMIN": True,
}


class AccountsSettings(object):
    """
    Based on https://github.com/encode/django-rest-framework/blob/master/rest_framework/settings.py
    """

    def __init__(self, settings=None, defaults=None):
        self.settings = settings or {}
        self.defaults = defaults or {}

    def __getattr__(self, attr):
        if attr not in self.defaults.keys():
            raise AttributeError("Invalid Penn Labs accounts setting: %s" % attr)

        try:
            val = self.settings[attr]
        except KeyError:
            val = self.defaults[attr]

        setattr(self, attr, val)
        return val


accounts_settings = AccountsSettings(USER_SETTINGS, DEFAULTS)



from django.urls import path

from accounts.views import CallbackView, LoginView, LogoutView


app_name = "accounts"


urlpatterns = [
    path("callback/", CallbackView.as_view(), name="callback"),
    path("login/", LoginView.as_view(), name="login"),
    path("logout/", LogoutView.as_view(), name="logout"),
]
